/*
 * Some portions of code inspired by M.H.Kay,
 * Saxon B 8.6.1 samples, http://saxon.sf.net.
 *  
 * Created on 6.2.2006 by Tomas Pitner
 *
 */
package cz.muni.fi.pb138.demo.xquery;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.Properties;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import net.sf.saxon.Configuration;
import net.sf.saxon.query.DynamicQueryContext;
import net.sf.saxon.query.StaticQueryContext;
import net.sf.saxon.query.XQueryExpression;
import net.sf.saxon.trans.XPathException;

public class Demo {

    public static void runQuery1() throws TransformerException {

        final Configuration config = new Configuration();
        final StaticQueryContext sqc = new StaticQueryContext(config);
        final XQueryExpression exp = sqc
                .compileQuery("<a xmlns='http://a/uri' xmlns:a='another.uri'>text</a>");

        final DynamicQueryContext dynamicContext = new DynamicQueryContext(
                config);
        DOMResult result = new DOMResult();
        exp.run(dynamicContext, result, new Properties());

        // now serialize the DOM

        Transformer identity = TransformerFactory.newInstance()
                .newTransformer();
        identity.transform(new DOMSource(result.getNode()), new StreamResult(
                System.out));
    }

    public static void runQuery1a() throws TransformerException {

        final Configuration config = new Configuration();
        final StaticQueryContext sqc = new StaticQueryContext(config);
        final XQueryExpression exp = sqc
                .compileQuery("<a xmlns='http://a/uri' xmlns:a='another.uri'>text</a>");

        final DynamicQueryContext dynamicContext = new DynamicQueryContext(
                config);

        exp.run(dynamicContext, new StreamResult(System.out), new Properties());
    }

    public static void runQuery2() throws XPathException {
        final Configuration config = new Configuration();
        final StaticQueryContext sqc = new StaticQueryContext(config);
        final XQueryExpression exp = sqc
                .compileQuery("avg(for $i in 1 to 10 return $i*$i)");
    
        final DynamicQueryContext dynamicContext = new DynamicQueryContext(
                config);
        final BigDecimal result = (BigDecimal) exp
                .evaluateSingle(dynamicContext);
        System.out.println(result);
    
    }

    public static void runQueryFromFile() throws XPathException, IOException {
        final Configuration config = new Configuration();
        final StaticQueryContext sqc = new StaticQueryContext(config);
        final XQueryExpression exp = sqc.compileQuery(new FileReader("demo-xquery.xq"));

        final DynamicQueryContext dynamicContext = new DynamicQueryContext(config);
        dynamicContext.setContextNode(sqc.buildDocument(new StreamSource("demo.xml")));
        
        final Properties props = new Properties();
        props.setProperty(OutputKeys.METHOD, "html");
        props.setProperty(OutputKeys.DOCTYPE_PUBLIC, "-//W3C//DTD HTML 4.01 Transitional//EN");
        
        exp.run(dynamicContext, new StreamResult(new File("demo-xquery-out.html")), props);

    }

    /**
     * @param args
     * @throws TransformerException 
     * @throws IOException 
     */
    public static void main(String[] args) throws TransformerException,
            IOException {

        System.out.println("--- run XQuery 1 ---");

        runQuery1();

        System.out.println("\n--- run XQuery 2 ---");

        runQuery2();
        
        System.out.println("\n--- run XQuery from file ---");

        runQueryFromFile();

        System.out.println("--- finished ---");
    }

}
